"""PyPI Setup For Audire Package"""

import os
import re
import setuptools


def _read(filename, version=False):
    with open(
        os.path.join(os.path.dirname(__file__), filename), encoding="utf8"
    ) as file:
        text = file.read()
    if version:
        return re.search(r'__version__ = "(.*?)"', text).group(1)
    return text


setuptools.setup(
    name="Audire",
    packages=setuptools.find_packages(),
    version=_read("audire/__init__.py", version=True),
    license="MIT",
    description="A Package to search and get download links for songs from various platforms asynchronously.",
    long_description=_read("README.md"),
    long_description_content_type="text/markdown",
    author="Eren",
    author_email="linuxguy312@segfault.net",
    url="https://github.com/LinuxGuy312/Audire",
    keywords=["songs", "song", "music", "youtube", "ytm", "Audire", "audire"],
    install_requires=["aiohttp", "bs4"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: MIT License",
    ],
    python_requires="~=3.7",
)



"""
Audire
Copyright (c) 2024 Eren

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


class DotDict(dict):
    """
    A class that parses dict into an object that can be accessed with dot notation.

    Args:
        dict (dict): The dictionary to convert to a dotdict.

    Returns:
        DotDict (dotdict): The dotdict representation of the dictionary.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        for key, value in self.items():
            if isinstance(value, dict):
                self[key] = DotDict(value)
            elif isinstance(value, list):
                self[key] = [
                    DotDict(item) if isinstance(item, dict) else item for item in value
                ]

    def __getattr__(self, attr):
        return self.get(attr, None)

    def __getitem__(self, key):
        return self.get(key, None)

    def __setattr__(self, key, value):
        self[key] = value

    def __delattr__(self, key):
        del self[key]

    def __setitem__(self, key, value):
        super().__setitem__(key, value)

    def __delitem__(self, key):
        super().__delitem__(key)



"""
Audire
Copyright (c) 2024 Eren

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


class BaseError(Exception):
    """
    The base class for all errors raised within this package.

    Attributes:
        error_message (str, optional): The specific error message.
    """

    message = "An error occurred."

    def __init__(self, error_message=None):
        self.error_message = error_message or self.message

    def __str__(self):
        return self.error_message

    def __repr__(self):
        return f"<{self.__class__.__name__} error_message='{self.error_message}'>"


class PlatformNotSupportedError(BaseError):
    """
    Exception raised when an unsupported platform is requested.
    """

    message = "Requested Platform is Not Supported."


class NoResultsFoundError(BaseError):
    """
    Exception raised when no results are found for a given query.
    """

    message = "No Results Found for the given query"
